#pragma once

#include <atlbase.h>
#include <atlwin.h>
#include "InstallController.h"
#include "util.h"

#define WM_SCHEDULE_PREPAREMODULE	(WM_USER+1024)
#define WM_SCHEDULE_INSTALLMODULE	(WM_USER+1025)
#define WM_PREPARE_RECEIVE			(WM_USER+1026)
#define WM_PREPARE_FINISH			(WM_USER+1027)
#define WM_INSTALL_FINISH			(WM_USER+1028)
#define WM_PREPARE_START			(WM_USER+1029)
#define WM_INSTALL_START			(WM_USER+1030)

struct DownloadSize
{
	DWORDLONG currentSize;
	DWORDLONG maxSize;
	TCHAR	  guid[GUID_SIZE];
};

class ScheduleModuleWnd : public CWindowImpl<ScheduleModuleWnd, CWindow, CNullTraits>
{
public:
	ScheduleModuleWnd(InstallController *installController)
	{
		m_installController = installController;
	}

	~ScheduleModuleWnd()
	{
		DestroyWindow();
	}

	// Optionally specify name of the new Windows class
	DECLARE_WND_CLASS(_T("ScheduleModuleWnd")) 
	// If this macro is not specified in your
	// class, ATL will generate a class name

	BEGIN_MSG_MAP(ScheduleModuleWnd)
		MESSAGE_HANDLER(WM_SCHEDULE_PREPAREMODULE, OnProcessPrepareModule)
		MESSAGE_HANDLER(WM_SCHEDULE_INSTALLMODULE, OnProcessInstallModule)
		MESSAGE_HANDLER(WM_PREPARE_RECEIVE, OnPrepareReceive)
		MESSAGE_HANDLER(WM_PREPARE_FINISH, OnPrepareFinsih)
		MESSAGE_HANDLER(WM_INSTALL_FINISH, OnInstallFinsih)
		MESSAGE_HANDLER(WM_PREPARE_START, OnPrepareStart)
		MESSAGE_HANDLER(WM_INSTALL_START, OnInstallStart)
	END_MSG_MAP()

	LRESULT OnProcessPrepareModule(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return m_installController->ProcessPrepare();
	}

	LRESULT OnProcessInstallModule(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		return m_installController->ProcessInstall();
	}

	LRESULT OnPrepareReceive(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		DownloadSize *item = (DownloadSize *)lParam;
		CALLBACK_RETURN ret =  m_installController->OnProgress(item->guid, item->currentSize, item->maxSize,
												CALLBACK_PROGRESS_STATUS(wParam));
		delete item;
		return 1;
	}

	LRESULT OnPrepareFinsih(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		CALLBACK_FINISHED_INFO *callbackInfo = reinterpret_cast<CALLBACK_FINISHED_INFO *>(lParam);
		if( NULL == callbackInfo )
			return 0;

		CALLBACK_RETURN ret = m_installController->OnPrepareFinished( callbackInfo->GetGuid(), callbackInfo->GetFinishedStatus(), callbackInfo->GetErrorInfo() );
		REMOVE_OBJECT(callbackInfo);

		MyPostMessage(m_hWnd, WM_SCHEDULE_PREPAREMODULE, 0, 0 );
		return 1;
	}

	LRESULT OnInstallFinsih(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		CALLBACK_FINISHED_INFO *callbackInfo = reinterpret_cast<CALLBACK_FINISHED_INFO *>(lParam);
		if( NULL == callbackInfo )
			return 0;

		CALLBACK_RETURN ret = m_installController->OnFinished( callbackInfo->GetGuid(), callbackInfo->GetFinishedStatus(), callbackInfo->GetErrorInfo() );

		REMOVE_OBJECT(callbackInfo);
		if ( ret != CALLBACK_RETURN_ALLDONE)
		{
			MyPostMessage(m_hWnd, WM_SCHEDULE_INSTALLMODULE, 0, 0 );
		}
		return 1;
	}

	LRESULT OnPrepareStart(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		const TCHAR *guid = (const TCHAR *)lParam;
		CALLBACK_RETURN ret = m_installController->OnPrepareStart( guid );
		return 1;
	}

	LRESULT OnInstallStart(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		const TCHAR *guid = (const TCHAR *)lParam;
		CALLBACK_RETURN ret = m_installController->OnInstallStart( guid );
		return 1;
	}
private:
	InstallController*		m_installController;
};

